unit uMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, FileCtrl, Buttons, uArt;

type
  TfrmArtView = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    DriveComboBox: TDriveComboBox;
    DirectoryListBox: TDirectoryListBox;
    Splitter1: TSplitter;
    Splitter2: TSplitter;
    FileListBox: TFileListBox;
    FilterComboBox: TFilterComboBox;
    lvImages: TListView;
    Splitter3: TSplitter;
    Panel3: TPanel;
    mmArtInfo: TMemo;
    Splitter4: TSplitter;
    SaveBmp: TSaveDialog;
    Panel4: TPanel;
    Panel5: TPanel;
    PaintBox: TPaintBox;
    cbPalette: TComboBox;
    btSave: TBitBtn;
    btSaveAll: TBitBtn;
    btLoad: TBitBtn;
    OpenBmp: TOpenDialog;
    SaveArt: TSaveDialog;
    BitBtn1: TBitBtn;
    procedure Panel2Resize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FileListBoxChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lvImagesChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure PaintBoxPaint(Sender: TObject);
    procedure cbPaletteChange(Sender: TObject);
    procedure btSaveClick(Sender: TObject);
    procedure btSaveAllClick(Sender: TObject);
    procedure btLoadClick(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
  protected
    ArtInfo: TArtInfo;
  public
  end;

var
  frmArtView: TfrmArtView;

implementation

{$R *.DFM}

procedure TfrmArtView.Panel2Resize(Sender: TObject);
begin
  DriveComboBox.Width := Panel2.Width;
  FilterComboBox.Width := Panel2.Width;
end;

procedure TfrmArtView.FormCreate(Sender: TObject);
begin
  FileListBox.Mask := FilterComboBox.Mask;
  ArtInfo := TArtInfo.Create;
  SaveBmp.InitialDir := ExtractFilePath(Application.ExeName);
  if DirectoryExists('C:\Program files\CDBoom') then
    DirectoryListBox.Directory := 'C:\Program files\CDBoom';
end;

procedure TfrmArtView.FileListBoxChange(Sender: TObject);

  function HeadToStr(StartI, EndI: Integer): string;
  var
    i: integer;
  begin
    Result := '';
    for i := StartI to EndI do
    begin
      if Result <> '' then
        Result := Result + ' ';
      Result := Result + IntToHex(ArtInfo.Head[i], 8);
    end;
  end;

var
  i, j, svePalIndx: integer;
  ImageInfo: PImageInfo;
  Item: TListItem;
begin
  lvImages.Items.Clear;
  mmArtInfo.Lines.Clear;
  svePalIndx := cbPalette.ItemIndex;
  cbPalette.Items.Clear;
  ArtInfo.LoadFile(FileListBox.FileName);
  if not ArtInfo.IsEmpty then
  begin
    mmArtInfo.Lines.Add('Positions: ' + IntToStr(ArtInfo.PictureCount));
    mmArtInfo.Lines.Add('Frames:    ' + IntToStr(ArtInfo.FrameCount));
    mmArtInfo.Lines.Add('Palettes:  ' + IntToStr(ArtInfo.PaletteCount));
    mmArtInfo.Lines.Add('Head:');
    i := 0;
    while i < $20 do
    begin
      mmArtInfo.Lines.Add(HeadToStr(i, i + 4));
      Inc(i, 4);
    end;
    mmArtInfo.Lines.Add(HeadToStr($20, $20));
    lvImages.Items.BeginUpdate;
    try
      for i := 0 to ArtInfo.PictureCount - 1 do
        for j := 0 to ArtInfo.FrameCount - 1 do
        begin
          ImageInfo := ArtInfo.GetImageInfoPtr(i, j);
          Item := lvImages.Items.Add;
          Item.Data := Pointer(i * ArtInfo.FrameCount + j);
          Item.Caption := IntToStr(i);
          Item.SubItems.Add(IntToStr(j));
          Item.SubItems.Add(IntToStr(ImageInfo.Width));
          Item.SubItems.Add(IntToStr(ImageInfo.Height));
          Item.SubItems.Add(IntToStr(ImageInfo.Size));
          Item.SubItems.Add(IntToStr(ImageInfo.Left));
          Item.SubItems.Add(IntToStr(ImageInfo.Top));
          Item.SubItems.Add(IntToStr(ImageInfo.DeltaX));
          Item.SubItems.Add(IntToStr(ImageInfo.DeltaY));
        end;
    finally
      lvImages.Items.EndUpdate
    end;
  end;
  for i := 0 to ArtInfo.PaletteCount - 1 do
    cbPalette.Items.Add(IntToStr(i));
  if (svePalIndx < 0) or (svePalIndx >= cbPalette.Items.Count) then
    cbPalette.ItemIndex := 0
  else
    cbPalette.ItemIndex := svePalIndx;
  if (lvImages.Items.Count > 0) and (lvImages.Selected = nil) then
    lvImages.Selected := lvImages.Items[0];
end;

procedure TfrmArtView.FormDestroy(Sender: TObject);
begin
  ArtInfo.Free;
end;

procedure TfrmArtView.lvImagesChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  PaintBox.Invalidate;
end;

procedure TfrmArtView.PaintBoxPaint(Sender: TObject);
var
  Bitmap: TBitmap;
  Stream: TMemoryStream;
begin
  if lvImages.Selected = nil then Exit;
  Stream := TMemoryStream.Create;
  try
    ArtInfo.CreateBitmapStream(Integer(lvImages.Selected.Data),
      cbPalette.ItemIndex, False, Stream);
    Bitmap := TBitmap.Create;
    try
      Bitmap.LoadFromStream(Stream);
      PaintBox.Canvas.CopyRect(Rect(10, 10, 10 + Bitmap.Width, 10 + Bitmap.Height),
        Bitmap.Canvas, Rect(0, 0, Bitmap.Width, Bitmap.Height));
    finally
      Bitmap.Free;
    end;
  finally
    Stream.Free;
  end;
end;

procedure TfrmArtView.cbPaletteChange(Sender: TObject);
begin
  PaintBox.Invalidate;
end;

procedure TfrmArtView.btSaveClick(Sender: TObject);
var
  Stream: TMemoryStream;
begin
  if lvImages.Selected = nil then Exit;
  if SaveBmp.Execute then
  begin
    SaveBmp.InitialDir := ExtractFilePath(SaveBmp.FileName);
    Stream := TMemoryStream.Create;
    try
      ArtInfo.CreateBitmapStream(Integer(lvImages.Selected.Data),
        cbPalette.ItemIndex, False, Stream);
      Stream.SaveToFile(SaveBmp.FileName);
    finally
      Stream.Free;
    end;
  end;
end;

procedure TfrmArtView.btSaveAllClick(Sender: TObject);
var
  i, j, k, N: Integer;
  Base: string;
  Stream: TMemoryStream;
begin
  if lvImages.Selected = nil then Exit;
  if ArtInfo.IsEmpty then Exit;
  if SaveBmp.Execute then
  begin
    SaveBmp.InitialDir := ExtractFilePath(SaveBmp.FileName);
    Base := ExtractFilePath(SaveBmp.FileName) + ExtractFileName(FileListBox.FileName);
    N := Length(ExtractFileExt(Base));
    Base := Copy(Base, 1, Length(Base) - N);
    Stream := TMemoryStream.Create;
    try
      for i := 0 to ArtInfo.PictureCount - 1 do
        for j := 0 to ArtInfo.FrameCount - 1 do
          for k := 0 to ArtInfo.PaletteCount - 1 do
          begin
            ArtInfo.CreateBitmapStream(i * ArtInfo.FrameCount + j, k, False, Stream);
            Stream.SaveToFile(Format('%s_%d_%d_%d.bmp', [Base, i, j, k]));
          end;
    finally
      Stream.Free;
    end;
  end;
end;

procedure TfrmArtView.btLoadClick(Sender: TObject);
var
  Stream: TMemoryStream;
begin
  if ArtInfo.IsEmpty then Exit;
  if lvImages.Selected = nil then Exit;
  if OpenBmp.Execute then
  begin
    Stream := TMemoryStream.Create;
    try
      Stream.LoadFromFile(OpenBmp.FileName);
      ArtInfo.ReplaceBitmap(Integer(lvImages.Selected.Data), cbPalette.ItemIndex,
        ArtInfo.FrameCount * ArtInfo.PictureCount = 1, Stream);
    finally
      Stream.Free;
    end;
    PaintBox.Invalidate;
  end;
end;

procedure TfrmArtView.BitBtn1Click(Sender: TObject);
begin
  if ArtInfo.IsEmpty then Exit;
  if SaveArt.Execute then
    ArtInfo.SaveFile(SaveArt.FileName);
end;

end.
